import { injectScripts, checkAllObjectValuesAreTrue } from "../../scriptInjector"

let hasBeenLoaded = false
const vueMaskLibrary = 'theMask'; // inputMask, theMask

/**
 * Loads VueMask Library from CDN
 * 
 * Usage:
	import VueMaskLoader from 'hs-dev-helper/js-toolkit/cdnLoaders/VueMask/VueMaskLoader'
	(async () => await VueMaskLoader.loadVueMaskScripts())()
 */
export const loader = class {

	vueMask = {
		ready : false,
		loaded: {
			script: false
		}
	}

	loadVueMaskScripts = ({ library, onload = null } = {}) => { 
		return new Promise((res, rej) => { 
			vueMaskLibrary = library || vueMaskLibrary

			const checkIfAllAreLoaded = () => {
				this.vueMask.ready = checkAllObjectValuesAreTrue(this.vueMask.loaded)
				if (this.vueMask.ready) {
					let payload = vueMaskLibrary == 'theMask' ? {VueMask: window.VueTheMask} : null
					// Fire Event
					var event = new CustomEvent('VueMaskLoaded', { detail : payload });
					window.dispatchEvent(event);
					// OnLoad Callback
					if (this.vueMask.loaded.script && !!onload && typeof onload == 'function') { 
						onload(payload)
					}
					// Flag
					hasBeenLoaded = true
					// Return Promise
					res(payload)
				}
			}

			// Scripts
			let scripts = []
			switch (vueMaskLibrary) {
				case 'theMask':
					scripts = [
						{
							//https://vuejs-tips.github.io/vue-the-mask/
							type: 'script',
							src: '//cdnjs.cloudflare.com/ajax/libs/vue-the-mask/0.11.1/vue-the-mask.min.js',
							precheck: () => (!!window.Vue && typeof window.Vue == 'function'),
							onload: () => {
								this.vueMask.loaded.script = (typeof window.VueTheMask == 'object')
								window.Vue.use(window.VueTheMask);
								checkIfAllAreLoaded()
							}
						}
					]
					break;
				case 'inputMask':
					scripts = [
						{
							//https://www.npmjs.com/package/inputmask
							type: 'script',
							src: '//cdnjs.cloudflare.com/ajax/libs/inputmask/4.0.9/inputmask/inputmask.min.js',
							precheck: () => (!!window.Vue && typeof window.Vue == 'function'),
							onload: () => {this.vueMask.loaded.script = true								
								var inputmaskPlugin = {
									install: function(Vue, options) {
										Vue.directive('mask', {
											bind: function(el, binding, vnode ) {
												Inputmask(binding.value, {
													showMaskOnHover: false,
													//autoUnmask : true,
													onKeyValidation: (e) => { el.dispatchEvent(new Event('input')); }
												}).mask(el);
											}
										});
									}
								};
								window.Vue.use(inputmaskPlugin)
								checkIfAllAreLoaded()
							}
						}
					]
					break;
			}
			
			// Inject
			if (!hasBeenLoaded) {
				injectScripts(scripts)
			}
		})
	}

}

/**
 * Load VueMask from Vue
 * 
 * Usage
	import { VueMaskMixin } from 'hs-dev-helper/js-toolkit/cdnLoaders/VueMask/VueMaskLoader';
	...
	mixins: [VueMaskMixin],
	...
	computed: {
		ready() { 
			return !!this.vueMask.ready
		},
		...
	}
*/
export const VueMaskMixin = {
	created() {
		window.addEventListener('VueMaskLoaded', () => {
			this.vueMask.ready = true
		});
	},
	beforeMount() {
		(async () => await (new loader).loadVueMaskScripts())()
	},
	data: () => ({
		vueMask : {ready: false}
	}),
}

export default new loader